@@ -5,7 +5,8 @@ package Interchange6::Cart::Product;
 use strict;
 use Moo;
 use Interchange6::Types;
-with 'Interchange6::Role::Costs';
+use Interchange6::Hook;
+with 'Interchange6::Role::Costs', 'Interchange6::Role::Hookable';
 
 use namespace::clean;
 
@@ -59,7 +60,7 @@ has name => (
 
 =head2 price
 
-Product price is required and a positive number or zero.
+Product price is required and a positive number.
 
 Price is required, because you want to maintain the price that was valid at the time of adding to the cart. Should the price in the shop change in the meantime, it will maintain this price.
 
@@ -67,7 +68,7 @@ Price is required, because you want to maintain the price that was valid at the
 
 has price => (
     is        => 'ro',
-    isa       => AnyOf [ PositiveNum, Zero ],
+    isa       => PositiveNum,
     required  => 1,
 );
 
@@ -79,7 +80,7 @@ Selling price is the price after group pricing, tier pricing or promotional disc
 
 has selling_price => (
     is        => 'rw',
-    isa       => Num,
+    isa       => PositiveNum,
     builder   => 1,
     lazy      => 1,
 );
@@ -89,24 +90,6 @@ sub _build_selling_price {
     return $self->price;
 }
 
-=head2 discount_percent
-
-This is the integer discount percentage calculated from the difference
-between L</price> and L</selling_price>. This attribute should not normally
-be set since as it is a calculated value.
-
-=cut
-
-has discount_percent => (
-    is => 'lazy',
-);
-
-sub _build_discount_percent {
-    my $self = shift;
-    return 0 if $self->price == $self->selling_price;
-    return int( ( $self->price - $self->selling_price ) / $self->price * 100 );
-}
-
 =head2 quantity
 
 Product quantity is optional and has to be a natural number greater
@@ -115,18 +98,11 @@ than zero. Default for quantity is 1.
 =cut
 
 has quantity => (
-    is      => 'ro',
+    is      => 'rw',
     isa     => AllOf [ PositiveNum, Int ],
     default => 1,
-    writer  => 'set_quantity',
 );
 
-after quantity => sub {
-    my $self = shift;
-    $self->clear_subtotal;
-    $self->clear_total;
-};
-
 =head2 sku
 
 Unique product identifier is required.
